if not is_game_single_player then return end

ACT_CB_WALKING = allocate_mario_action(ACT_GROUP_MOVING | ACT_FLAG_MOVING)
ACT_CB_IDLE = allocate_mario_action(ACT_GROUP_MOVING | ACT_FLAG_MOVING)
ACT_CB_FALLING = allocate_mario_action(ACT_GROUP_MOVING | ACT_FLAG_MOVING)

local timer = 0
local speed_multiplier = 1
local sprint_speed = 2
local footstep_timer = 0

custom_mario_state_fields = {
    sideVel = 0,
    sprintTimer = 256,
    inactive = false,
}

set_first_person_enabled(true)

local footstep_sound_table = {
    [1] = {
        [1] = audio_sample_load('default_step_1.ogg'),
        [2] = audio_sample_load('default_step_2.ogg'),
        [3] = audio_sample_load('default_step_3.ogg'),
        [4] = audio_sample_load('default_step_4.ogg'),
    },
    [2] = {
        [1] = audio_sample_load('metal_step_1.ogg'),
        [2] = audio_sample_load('metal_step_2.ogg'),
        [3] = audio_sample_load('metal_step_3.ogg'),
        [4] = audio_sample_load('metal_step_4.ogg'),
    },
}

local function play_footstep_sound(m, vol)
    local footstep_sound = 1

    if gNetworkPlayers[m.playerIndex].currLevelNum == LEVEL_TTM then
        footstep_sound = 2
    end

    audio_sample_play(footstep_sound_table[footstep_sound][math.random(1, 4)], m.pos, clampf(vol, 0, 1))
end


local function act_cb_walking_and_idle(m)
    if custom_mario_state_fields.inactive then return end

    gFirstPersonCamera.offset.y = 128 + math.sin((footstep_timer * (math.pi * 2)) / (20 / speed_multiplier)) * ((m.forwardVel + custom_mario_state_fields.sideVel) / 5)

    if footstep_timer >= (20 / speed_multiplier) then
        play_footstep_sound(m, (math.abs(m.forwardVel) + math.abs(custom_mario_state_fields.sideVel) / 100))
        footstep_timer = 0
    end

    footstep_timer = footstep_timer + 1

    perform_ground_step(m)
end

local function act_cb_falling(m)
    if custom_mario_state_fields.inactive then return end

    perform_air_step(m, 0)
end

local function mario_update(m)
    if m ~= gMarioStates[0] then return end

    if custom_mario_state_fields.inactive then
        set_first_person_enabled(false)
        camera_freeze()
    else
        set_first_person_enabled(true)
        camera_unfreeze()
    end

    if custom_mario_state_fields.inactive then return end

    m.faceAngle.y = gFirstPersonCamera.yaw

    m.forwardVel = approach_f32(m.forwardVel, (-m.controller.stickY / 2) * speed_multiplier, 10, 10)
    custom_mario_state_fields.sideVel = approach_f32(custom_mario_state_fields.sideVel, (-m.controller.stickX / 2) * speed_multiplier, 10, 10)

    m.vel.x = (m.forwardVel * sins(m.faceAngle.y)) + -(custom_mario_state_fields.sideVel * coss(m.faceAngle.y))
    m.vel.z = (custom_mario_state_fields.sideVel * sins(m.faceAngle.y)) + (m.forwardVel * coss(m.faceAngle.y))

    timer = timer + 1

    if m.floorHeight ~= m.pos.y then
        set_mario_action(m, ACT_CB_FALLING, 1)
    else
        set_mario_action(m, ACT_CB_WALKING, 1)
    end

    if (m.controller.buttonDown & Z_TRIG) ~= 0 and custom_mario_state_fields.sprintTimer > 0 and m.action == ACT_CB_WALKING then
        speed_multiplier = sprint_speed
        custom_mario_state_fields.sprintTimer = custom_mario_state_fields.sprintTimer - 1
    else
        speed_multiplier = 1

        if custom_mario_state_fields.sprintTimer < 256 and m.controller.stickMag == 0 then
            custom_mario_state_fields.sprintTimer = custom_mario_state_fields.sprintTimer + 1
        elseif custom_mario_state_fields.sprintTimer < 256 and (m.controller.buttonDown & Z_TRIG) == 0 then
            custom_mario_state_fields.sprintTimer = custom_mario_state_fields.sprintTimer + 0.5
        end
    end
end

hook_mario_action(ACT_CB_WALKING, act_cb_walking_and_idle)
hook_mario_action(ACT_CB_FALLING, act_cb_falling)
hook_event(HOOK_MARIO_UPDATE, mario_update)