if not is_game_single_player then return end

E_MODEL_GOLDEN_SIGMA = smlua_model_util_get_id("golden_sigma_geo")

E_MODEL_ROBLOX_BOMB = smlua_model_util_get_id("roblox_bomb_geo")
E_MODEL_QUESTION_BLOCK = smlua_model_util_get_id("question_block_geo")
E_MODEL_TROLLFACE = smlua_model_util_get_id("trollface_geo")

SOUND_CLICK_FAST = audio_sample_load("click_fast.ogg")

SOUND_GOLDEN_SIGMA_DAMAGE_1 = audio_sample_load('golden_sigma_damage_1.ogg')
SOUND_GOLDEN_SIGMA_DAMAGE_2 = audio_sample_load('golden_sigma_damage_2.ogg')
SOUND_GOLDEN_SIGMA_DAMAGE_3 = audio_sample_load('golden_sigma_damage_3.ogg')

SOUND_TROLLFACE_DIE = audio_sample_load("trollface_jumpscare.ogg")

number_of_attacks = 2

golden_sigma_battle_in_session = false

local function point_within_range(vec3f, min_x, max_x, min_z, max_z)
    if vec3f.x > min_x then
        if vec3f.x < max_x then
            if vec3f.z > min_z then
                if vec3f.z < max_z then
                    return true
                end
            end
        end

        return false
    end
end

function start_anim_default(o, anim)
    smlua_anim_util_set_animation(o, anim)
    o.header.gfx.animInfo.animYTrans = 3
    o.header.gfx.animInfo.animAccel = 0
    o.header.gfx.animInfo.animFrame = o.header.gfx.animInfo.curAnim.loopStart
end

local function bhv_golden_sigma_init(o)
    o.oFlags = OBJ_FLAG_UPDATE_GFX_POS_AND_ANGLE
    o.oCollisionDistance = 7000
    o.oAngleVelYaw = 0
    o.header.gfx.skipInViewCheck = true

    o.oHomeX = o.oPosX
    o.oHomeY = o.oPosY
    o.oHomeZ = o.oPosZ

    o.oMaxHealth = 512
    o.oHealth = o.oMaxHealth
    o.oHurtCounter = 0

    obj_set_hitbox_radius_and_height(o, 128, 256)

    start_anim_default(o, 'golden_sigma_idle')
    obj_init_animation(o, 1)

    obj_scale(o, 2)
end

local function bhv_golden_sigma_loop(o)
    local m = nearest_mario_state_to_object(o)
    local bomb = obj_get_nearest_object_with_behavior_id(o, id_bhvRobloxBomb)

    if o.oAction == 0 then
        if o.oTimer == 0 then
            start_anim_default(o, 'golden_sigma_idle')
        end

        o.oFaceAngleYaw = approach_s16_asymptotic(o.oFaceAngleYaw, calculate_yaw({x = o.oPosX, y = o.oPosY, z = o.oPosZ}, m.pos) + 16384, 0.15)

        o.oPosX = approach_f32_asymptotic(o.oPosX, o.oHomeX, 0.05)
        o.oPosY = approach_f32_asymptotic(o.oPosY, o.oHomeY, 0.05)
        o.oPosZ = approach_f32_asymptotic(o.oPosZ, o.oHomeZ, 0.05)

        if (o.oTimer % 60) == 0 and golden_sigma_battle_in_session then
            o.oAction = math.random(1, number_of_attacks)
        end

        o.oGeneralRepeatCounter = 0
    elseif o.oAction == 1 then
        if o.oTimer == 0 then
            start_anim_default(o, 'golden_sigma_throw')
        end

        o.oFaceAngleYaw = approach_s16_asymptotic(o.oFaceAngleYaw, calculate_yaw({x = o.oPosX, y = o.oPosY, z = o.oPosZ}, m.pos) + 16384, 0.15)

        if (o.oTimer % 10) == 0 then
            local random_x = math.random(-1900, 1900)
            local random_z = math.random(-1900, 1900)

            spawn_non_sync_object(id_bhvRobloxBomb, E_MODEL_ROBLOX_BOMB, random_x, 0, random_z, nil)
        end

        if o.oTimer >= 38 then
            o.oAction = 0
        end
    elseif o.oAction == 2 then
        if o.oTimer == 0 then
            o.oGeneralRepeatCounter = o.oGeneralRepeatCounter + 1
            start_anim_default(o, 'golden_sigma_spawn_block')
        end

        o.oFaceAngleYaw = calculate_yaw({x = o.oPosX, y = o.oPosY, z = o.oPosZ}, m.pos) + 16384

        if o.oTimer == 15 then
            spawn_non_sync_object(id_bhvQuestionBlock, E_MODEL_QUESTION_BLOCK, o.oPosX, o.oPosY, o.oPosZ, nil)
        end

        if o.oTimer >= 38 then
            if o.oGeneralRepeatCounter >= 5 then
                o.oAction = 0
            else
                o.oTimer = -1
            end
        end
    elseif o.oAction == 10 then
        if o.oTimer == 0 then
            start_anim_default(o, 'golden_sigma_hurt')

            local random = math.random(1, 3)

            if random == 1 then
                audio_sample_play(SOUND_GOLDEN_SIGMA_DAMAGE_1, {x = o.oPosX, y = o.oPosY, z = o.oPosZ}, 1)
            elseif random == 2 then
                audio_sample_play(SOUND_GOLDEN_SIGMA_DAMAGE_2, {x = o.oPosX, y = o.oPosY, z = o.oPosZ}, 1)
            elseif random == 3 then
                audio_sample_play(SOUND_GOLDEN_SIGMA_DAMAGE_3, {x = o.oPosX, y = o.oPosY, z = o.oPosZ}, 1)
            end

            o.oHealth = o.oHealth - 24

            o.oForwardVel = -256

            o.oMoveAngleYaw = (o.oFaceAngleYaw + 16384) - 32768

            o.oHurtCounter = 1

            obj_move_xyz_using_fvel_and_yaw(o)
        end

        if o.oTimer >= 38 then
            o.oAction = 0
        end
    elseif o.oAction == 8 then
        if o.oTimer == 0 then
            start_anim_default(o, 'golden_sigma_idle')
        end
    end

    if bomb then
        if obj_check_hitbox_overlap(o, bomb) then
            obj_mark_for_deletion(bomb)
            spawn_non_sync_object(id_bhvExplosion, E_MODEL_EXPLOSION, o.oPosX, o.oPosY, o.oPosZ, nil)
            o.oAction = 10
        end
    end

    if o.oHurtCounter <= 0 then
        o.oHurtCounter = 0
    else
        o.oHurtCounter = o.oHurtCounter - 0.05
    end

    if o.oHealth <= 0 then
        cutscene_start(3)
    end
end

local function bhv_roblox_bomb_init(o)
    o.oFlags = OBJ_FLAG_UPDATE_GFX_POS_AND_ANGLE
    o.oCollisionDistance = 7000
    o.oAngleVelYaw = 0
    o.header.gfx.skipInViewCheck = true

    o.oHomeX = o.oPosX
    o.oHomeY = o.oPosY
    o.oHomeZ = o.oPosZ

    o.oBombTickSpeed = 16

    o.oBounciness = -0.5
    o.oGravity = -2
    o.oVelY = 15

    audio_sample_play(SOUND_CLICK_FAST, {x = o.oPosX, y = o.oPosY, z = o.oPosZ}, 1)

    obj_scale(o, 0.5)
end

local function bhv_roblox_bomb_loop(o)
    local nearest_mario = nearest_mario_state_to_object(o)

    if o.oBombTickSpeed <= 0 then
        spawn_non_sync_object(id_bhvExplosion, E_MODEL_EXPLOSION, o.oPosX, o.oPosY + 64, o.oPosZ, nil)
        obj_mark_for_deletion(o)
    end

    if o.oTimer >= o.oBombTickSpeed - 5 then
        o.oAnimState = 1
    else
        o.oAnimState = 0
    end

    if o.oTimer >= o.oBombTickSpeed then
        o.oTimer = 0
        o.oBombTickSpeed = o.oBombTickSpeed - 1

        audio_sample_play(SOUND_CLICK_FAST, {x = o.oPosX, y = o.oPosY, z = o.oPosZ}, 1)
    end

    if o.oPosY >= o.oFloorHeight + 5 then
        if lateral_dist_between_objects(o, nearest_mario.marioObj) < 512 and (nearest_mario.controller.buttonPressed & A_BUTTON) ~= 0 then
            local gs = obj_get_first_with_behavior_id(id_bhvGoldenSigma)

            o.oForwardVel = 64
            o.oVelY = math.random(35, 45)

            obj_rotate_towards_point(o, {x = gs.oPosX, y = gs.oPosY, z = gs.oPosZ}, 0, 0, 1, 1)
        end
    end

    cur_obj_move_standard(-70)
end

local function bhv_question_block_init(o)
    o.oFlags = OBJ_FLAG_UPDATE_GFX_POS_AND_ANGLE
    o.header.gfx.skipInViewCheck = true

    local m = nearest_mario_state_to_object(o)

    o.oBounciness = 0
    o.oGravity = -2
    o.oVelY = 15

    o.oForwardVel = 15 + (dist_between_object_and_point(o, m.pos.x, m.pos.y, m.pos.z) / 128)
    o.oMoveAnglePitch = obj_angle_to_object(o, m.marioObj)
    o.oMoveAngleYaw = obj_angle_to_object(o, m.marioObj)

    cur_obj_update_floor_height_and_get_floor()
end

local function bhv_question_block_loop(o)
    cur_obj_update_floor_height_and_get_floor()

    o.oFaceAngleYaw = o.oFaceAngleYaw + 1024
    o.oFaceAnglePitch = o.oFaceAnglePitch + 1024
    o.oFaceAngleRoll = math.sin(o.oTimer / 4) * 4096

    cur_obj_move_using_fvel_and_gravity()
    cur_obj_move_using_vel()
    cur_obj_update_floor_height_and_get_floor()

    if (o.oPosY <= o.oFloorHeight) or (o.oPosY <= 0 and point_within_range({x = o.oPosX, y = o.oPosY, z = o.oPosZ}, -2000, 2000, -2000, 2000)) then
        local lottery = {
            [1] = {bhv = id_bhvTrollface, model = E_MODEL_TROLLFACE, special_param = 1},
            [2] = {bhv = id_bhvRobloxBomb, model = E_MODEL_ROBLOX_BOMB, special_param = 1},
        }

        local random_obj = lottery[math.random(1, #lottery)]

        spawn_non_sync_object(random_obj.bhv, random_obj.model, o.oPosX, o.oPosY + 192, o.oPosZ, function(obj)
            obj.oAction = random_obj.special_param
        end)

        audio_sample_play(SOUND_SMB_BLOCK_BREAK, {x = o.oPosX, y = o.oPosY, z = o.oPosZ}, 1)
        obj_mark_for_deletion(o)
    end

    if (o.oTimer % 5) == 0 then
        if o.oAnimState ~= 3 then
            o.oAnimState = o.oAnimState + 1
        else
            o.oAnimState = 1
        end
    end
end

local function bhv_trollface_init(o)
    o.oFlags = OBJ_FLAG_UPDATE_GFX_POS_AND_ANGLE
    o.oMoveFlags = o.oMoveFlags | OBJ_MOVE_IN_AIR
	o.header.gfx.node.flags = o.header.gfx.node.flags | GRAPH_RENDER_BILLBOARD

	o.header.gfx.skipInViewCheck = true

    o.oInteractType = INTERACT_WATER_RING

    cur_obj_set_hitbox_radius_and_height(96, 96)
	cur_obj_scale(1)
end

local function bhv_trollface_loop(o)
    o.oGraphYOffset = math.sin(o.oTimer * 0.2) * 30

    if o.oAction ~= 8 then
        local nm = nearest_mario_state_to_object(o)

        o.oForwardVel = 15 * (dist_between_objects(o, nm.marioObj) / 325)

        obj_rotate_towards_point(o, nm.pos, 0, 0, 0, 0)

        if obj_check_hitbox_overlap(o, nm.marioObj) then
            audio_sample_play(SOUND_TROLLFACE_DIE, nm.pos, 1.2)

            create_troll_jumpscare()

            nm.hurtCounter = 16

            obj_mark_for_deletion(o)
        end

        if o.oTimer > 150 then
            obj_mark_for_deletion(o)
        end

        obj_compute_vel_from_move_pitch(o.oForwardVel)
        obj_move_xyz_using_fvel_and_yaw(o)
    end
end

id_bhvGoldenSigma = hook_behavior(nil, OBJ_LIST_GENACTOR, true, bhv_golden_sigma_init, bhv_golden_sigma_loop, 'bhvGoldenSigma')
id_bhvRobloxBomb = hook_behavior(nil, OBJ_LIST_GENACTOR, true, bhv_roblox_bomb_init, bhv_roblox_bomb_loop, 'bhvRobloxBomb')
id_bhvQuestionBlock = hook_behavior(nil, OBJ_LIST_GENACTOR, true, bhv_question_block_init, bhv_question_block_loop, 'bhvQuestionBlock')
id_bhvTrollface = hook_behavior(nil, OBJ_LIST_SURFACE, true, bhv_trollface_init, bhv_trollface_loop, 'bhvTrollface')