if not is_game_single_player then return end

local CROWBAR_BODYHIT_VOLUME = 128
local CROWBAR_WALLHIT_VOLUME = 512
local cbar_stage = 1

local cbar_miss = audio_sample_load("cbar_miss1.ogg")
local last_known_pos = {}
local last_air_anim = 0

E_MODEL_CROWBAR = smlua_model_util_get_id("crowbar_geo")

local crowbar_anims = {
    {anim = 'crowbar_anim_attack1miss', frames = 10},
    {anim = 'crowbar_anim_attack2', frames = 13},
    {anim = 'crowbar_anim_attack3', frames = 18},
    {anim = 'crowbar_anim_attack1', frames = 10},
    {anim = 'crowbar_anim_attack2hit', frames = 13},
    {anim = 'crowbar_anim_attack3hit', frames = 18},
    {anim = 'crowbar_anim_idle1', frames = 35},
    {anim = 'crowbar_anim_idle2', frames = 80},
    {anim = 'crowbar_anim_idle3', frames = 80},
    {anim = 'crowbar_anim_draw', frames = 12},
    {anim = 'crowbar_anim_holster', frames = 12},
    {anim = 'crowbar_anim_none', frames = 2},
}

local soft_materials = {
    [41] = true,
    [19] = true,
}

local function start_animation(obj, anim, table)
    smlua_anim_util_set_animation(obj, table[anim].anim)
    frame_countdown = table[anim].frames
    obj.header.gfx.animInfo.animYTrans = 3
    obj.header.gfx.animInfo.animAccel = 0
    obj.header.gfx.animInfo.animFrame = obj.header.gfx.animInfo.curAnim.loopStart
end

local function swing_crowbar()
    local m = gMarioStates[0]
    local crowbar = obj_get_first_with_behavior_id(id_bhvWeapon)
    local hitpos = raycast_forward()

    if cbar_stage == 1 then
        cbar_stage = 2
    else
        cbar_stage = 1
    end

    if vec3f_dist(gLakituState.pos, hitpos.hitPos) < 256 then
        start_animation(crowbar, cbar_stage + 4, crowbar_anims)

        if soft_materials[hitpos.surface.type] then
            audio_sample_play(audio_sample_load("cbar_hit" .. tostring(math.random(1,2) .. ".ogg")), m.pos, 0.25)
            play_sound(SOUND_GENERAL_SOFT_LANDING, hitpos.hitPos)
        else
            audio_sample_play(audio_sample_load("cbar_hit" .. tostring(math.random(1,2) .. ".ogg")), m.pos, 1)
        end

        if hitpos.surface.object then
            if obj_has_behavior_id(hitpos.surface.object, id_bhvWegaMachine1) then
                hitpos.surface.object.oHealth = hitpos.surface.object.oHealth - 16
            end
        end
    else
        current_anims_available = {1, 2, 3}

        table.remove(current_anims_available, current_anims_available[last_air_anim])

        local air_anim = current_anims_available[math.random(1, #current_anims_available)]

        start_animation(crowbar, air_anim, crowbar_anims)

        last_air_anim = air_anim
    end

    audio_sample_play(cbar_miss, m.pos, 1)
end

local function mario_update(m)
    local crowbar = obj_get_first_with_behavior_id(id_bhvWeapon)

    if crowbar then
        if m.controller.buttonPressed & A_BUTTON ~= 0 then
            swing_crowbar()
        end

        if frame_countdown == 0 then
            start_animation(crowbar, 12, crowbar_anims)
        end
    end
end

hook_event(HOOK_MARIO_UPDATE, mario_update)