if not is_game_single_player then return end

E_MODEL_WEGA_DOOR = smlua_model_util_get_id("wega_door_geo")
E_MODEL_RETRO_PIPE = smlua_model_util_get_id("retro_pipe_geo")
E_MODEL_BLUE_SHINE = smlua_model_util_get_id("blue_shine_geo")
E_MODEL_TROLL_BOMB = smlua_model_util_get_id("troll_bomb_geo")

COL_WEGA_DOOR = smlua_collision_util_get("wega_door_collision")

-- Wega Machines

E_MODEL_WEGA_MACHINE_1 = smlua_model_util_get_id("pc_geo")
COL_WEGA_MACHINE_1 = smlua_collision_util_get("pc_collision")

E_MODEL_WEGA_MACHINE_2 = smlua_model_util_get_id("machine_2_geo")
COL_WEGA_MACHINE_2 = smlua_collision_util_get("machine_2_collision")
E_MODEL_WEGA_MACHINE_2_BROKEN = smlua_model_util_get_id("machine_2_broken_geo")
COL_WEGA_MACHINE_2_BROKEN = smlua_collision_util_get("machine_2_broken_collision")

E_MODEL_WEGA_MACHINE_3 = smlua_model_util_get_id("machine_3_geo")
COL_WEGA_MACHINE_3 = smlua_collision_util_get("machine_3_collision")

E_MODEL_MACHINE_GIB = smlua_model_util_get_id("machine_gib_geo")

E_MODEL_MONITOR = smlua_model_util_get_id("monitor_geo")

E_MODEL_BREAK_DOOR = smlua_model_util_get_id("break_door_geo")

E_MODEL_ZESTY_BAR = smlua_model_util_get_id("zesty_bar_geo")

E_MODEL_FOG = smlua_model_util_get_id("fog_geo")

SOUND_EXPLODE_1 = audio_sample_load("explode1.ogg")
SOUND_EXPLODE_2 = audio_sample_load("explode2.ogg")
SOUND_BREAK_COMPUTER = audio_sample_load("breakcomputer.ogg")
SOUND_ZESTY_BAR = audio_sample_load("zesty_bar.ogg")

SOUND_TROLL_BOMB_EXPLODE_1 = audio_sample_load("troll_bomb_1.ogg")
SOUND_TROLL_BOMB_EXPLODE_2 = audio_sample_load("troll_bomb_2.ogg")
SOUND_TROLL_BOMB_EXPLODE_3 = audio_sample_load("troll_bomb_3.ogg")
SOUND_TROLL_BOMB_SPAWN = audio_sample_load("troll_bomb_spawn.ogg")

machines_destroyed = 0

define_custom_obj_fields({
    oHurtCounter = "f32",
    oMaxHealth = "s32",
    oScale = 'f32',
    oBombTickSpeed = 's32',
    oGeneralRepeatCounter = 's32',
})

local function machine_gib_init(o)
    o.oFlags = OBJ_FLAG_UPDATE_GFX_POS_AND_ANGLE
    o.header.gfx.skipInViewCheck = true

    local randomfvel = math.random(1, 20)
    local random = math.random(20, 50)
    local randomyaw = math.random(1, 65536)

    o.oBounciness = 0
    o.oGravity = -4
    o.oVelY = random
    o.oPosY = o.oPosY + 10

    if o.oBehParams2ndByte ~= 1 then
        o.oForwardVel = randomfvel
        o.oFaceAngleYaw = randomyaw
        o.oMoveAngleYaw = o.oFaceAngleYaw
    end

    cur_obj_update_floor_height_and_get_floor()
end

local function machine_gib_loop(o)
    local m = gMarioStates[0]
    local random = math.random(1, 1500)

    if m.marioObj.oTimer < 10 then
        obj_mark_for_deletion(o)
    end

    cur_obj_update_floor_height_and_get_floor()
    if o.oPosY > o.oFloorHeight then
        cur_obj_move_using_fvel_and_gravity()
        cur_obj_move_using_vel()
        cur_obj_update_floor_height_and_get_floor()
        o.oFaceAnglePitch = o.oFaceAnglePitch + random
        o.oFaceAngleRoll = o.oFaceAngleRoll + random
        o.oFaceAngleYaw = o.oFaceAngleYaw + random
    else
        o.oPosY = o.oFloorHeight
    end

    if o.oTimer >= 100 then
        local new_scale = approach_f32_asymptotic(o.oScale, 0, 0.15)

        obj_scale(o, new_scale)
        o.oScale = new_scale
    end

    if o.oTimer > 150 then
        obj_mark_for_deletion(o)
    end
end

local function summon_gibs(o)
    local m = gMarioStates[0]

    if m.playerIndex ~= 0 then return end

    for i = 0, 30 do
        spawn_non_sync_object(id_bhvMachineGib, E_MODEL_MACHINE_GIB, o.oPosX, o.oPosY, o.oPosZ, function (gib)
            local random = math.random() + 0.5
            obj_scale(gib, random)
            gib.oScale = random
        end)
    end
end

local function summon_special_gibs(pos, radius)
    local m = gMarioStates[0]

    if m.playerIndex ~= 0 then return end

    for i = 0, 30 do
        spawn_non_sync_object(id_bhvMachineGib, E_MODEL_MACHINE_GIB, pos.x + ((math.random() - 0.5) * 2) * radius, pos.y + math.random() * radius, pos.z, function (gib)
            local random = math.random() + 0.5
            obj_scale(gib, random)
            gib.oScale = random
            gib.oForwardVel = obj_get_first_with_behavior_id(id_bhvWega).oForwardVel * 5
            gib.oFaceAngleYaw =  obj_get_first_with_behavior_id(id_bhvBreakDoor).oFaceAngleYaw - 16384
            gib.oMoveAngleYaw =  obj_get_first_with_behavior_id(id_bhvBreakDoor).oMoveAngleYaw - 16384
        end)
    end
end

local function bhv_wega_door_init(o)
    o.oFlags = OBJ_FLAG_UPDATE_GFX_POS_AND_ANGLE
    o.oCollisionDistance = 7000
    o.collisionData = COL_WEGA_DOOR
    o.oAngleVelYaw = 0
    o.header.gfx.skipInViewCheck = true

    o.oHomeX = o.oPosX
    o.oHomeY = o.oPosY
    o.oHomeZ = o.oPosZ
end

local function bhv_wega_door_loop(o)
    local m = gMarioStates[0]

    load_object_collision_model()

    if o.oAction == 0 then
        if (m.controller.buttonPressed & A_BUTTON) ~= 0 then
            local screen_coords = {x=0,y=0,z=0}
            local width = djui_hud_get_screen_width()

            djui_hud_world_pos_to_screen_pos({x=o.oPosX,y=o.oPosY,z=o.oPosZ}, screen_coords)

            if screen_coords.x < width and screen_coords.x > 0 and dist_between_objects(m.marioObj, o) < 512 then
                audio_sample_play(SOUND_DOOR_MOVE, {x = o.oPosX, y = o.oPosY, z = o.oPosZ}, 1)
                o.oAction = 1
            end
        end
    elseif o.oAction == 1 then
        o.oPosY = approach_f32(o.oPosY, o.oHomeY + 475, 480 / 60, 480 / 60)

        if o.oTimer >= 60 then
            o.oAction = 2
        end
    elseif o.oAction == 2 then
        if o.oTimer >= 300 then
            audio_sample_play(SOUND_DOOR_MOVE, {x = o.oPosX, y = o.oPosY, z = o.oPosZ}, 1)
            o.oAction = 3
        end
    elseif o.oAction == 3 then
        o.oPosY = approach_f32(o.oPosY, o.oHomeY, 480 / 60, 480 / 60)

        if o.oTimer >= 60 then
            o.oAction = 0
        end
    end
end

local function bhv_blue_shine_init(o)
    o.oFlags = OBJ_FLAG_UPDATE_GFX_POS_AND_ANGLE
    o.oFaceAngleYaw = calculate_yaw({x=o.oPosX,y=o.oPosY,z=o.oPosZ}, gLakituState.pos)
end

local function bhv_blue_shine_loop(o)
    o.oFaceAngleYaw = calculate_yaw({x=o.oPosX,y=o.oPosY,z=o.oPosZ}, gLakituState.pos)
end

local function bhv_wega_warp_init(o)
    cur_obj_set_hitbox_radius_and_height(128, 128)
    o.oAction = 0
end

local function bhv_wega_warp_loop(o)
    local m = gMarioStates[0]

    if o.oAction == 0 then
        if obj_check_hitbox_overlap(m.marioObj, o) then
            play_transition(WARP_TRANSITION_FADE_INTO_CIRCLE, 15, 0, 0, 0)

            o.oAction = 1
        end
    elseif o.oAction == 1 then
        if o.oTimer >= 15 then
            if o.oBehParams2ndByte == 1 then
                warp_to_level(LEVEL_BITDW, 1, 1)
            elseif o.oBehParams2ndByte == 2 then
                warp_to_level(LEVEL_TTM, 1, 1)
            elseif o.oBehParams2ndByte == 3 then
                warp_to_level(LEVEL_BBH, 1, 1)
            end

            audio_stream_stop(CASTLE_SONG)
        end
    end
end

local function break_machine(o)
    local random_break_sound = math.random(1, 2)

    if random_break_sound == 1 then
        audio_sample_play(SOUND_EXPLODE_1, {x = o.oPosX, y = o.oPosY, z = o.oPosZ}, 0.75)
    else
        audio_sample_play(SOUND_EXPLODE_2, {x = o.oPosX, y = o.oPosY, z = o.oPosZ}, 0.75)
    end

    spawn_non_sync_object(id_bhvBowserBombExplosion, E_MODEL_EXPLOSION, o.oPosX, o.oPosY, o.oPosZ, nil)

    summon_gibs(o)

    obj_mark_for_deletion(o)
end

local function break_machine_2(o)
    audio_sample_play(SOUND_BREAK_COMPUTER, {x = o.oPosX, y = o.oPosY, z = o.oPosZ}, 1.25)

    spawn_non_sync_object(id_bhvBowserBombExplosion, E_MODEL_EXPLOSION, o.oPosX, o.oPosY, o.oPosZ, nil)

    summon_gibs(o)

    obj_mark_for_deletion(o)
end

local function bhv_wega_machine_1_init(o)
    o.oFlags = OBJ_FLAG_UPDATE_GFX_POS_AND_ANGLE
	o.header.gfx.skipInViewCheck = true
    o.collisionData = COL_WEGA_MACHINE_1
    o.oHealth = 16

	cur_obj_scale(0.80)
end

local function bhv_wega_machine_1_loop(o)
    load_object_collision_model()

    if o.oHealth <= 0 then
        machines_destroyed = machines_destroyed + 1
        break_machine(o)
    end
end

local function bhv_wega_machine_2_init(o)
    o.oFlags = OBJ_FLAG_UPDATE_GFX_POS_AND_ANGLE
	o.header.gfx.skipInViewCheck = true
    o.collisionData = COL_WEGA_MACHINE_2
    o.oHealth = 32

	cur_obj_scale(2)
end

local function bhv_wega_machine_2_loop(o)
    load_object_collision_model()

    if o.oHealth <= 16 then
        obj_set_model_extended(o, E_MODEL_WEGA_MACHINE_2_BROKEN)
    end

    if o.oHealth <= 0 then
        machines_destroyed = machines_destroyed + 1
        break_machine(o)
    end
end

local function bhv_wega_machine_3_init(o)
    o.oFlags = OBJ_FLAG_UPDATE_GFX_POS_AND_ANGLE
	o.header.gfx.skipInViewCheck = true
    o.collisionData = COL_WEGA_MACHINE_3
    o.oHealth = 48

	cur_obj_scale(1)
end

local function bhv_wega_machine_3_loop(o)
    load_object_collision_model()

    if o.oHealth <= 0 then
        machines_destroyed = machines_destroyed + 1
        break_machine_2(o)
    end
end

function wrap_number(num, min, max)
    return (num - min) % max + min
end

local function bhv_wega_machine_spawner_loop(o)
    local machines = {
        [1] = {
            bhv = id_bhvWegaMachine1,
            model = E_MODEL_WEGA_MACHINE_1,
        },
        [2] = {
            bhv = id_bhvWegaMachine2,
            model = E_MODEL_WEGA_MACHINE_2,
        }
    }

    if gNetworkPlayers[0].currAreaSyncValid then
        local thing = wrap_number(o.oBehParams & 0xFF, 1, 15)
        local spawn_number = string.sub(gGlobalSyncTable.seed, thing, thing)

        if spawn_number == '' or spawn_number == nil then
            spawn_number = '0'
        end

        local machine_number = wrap_number(tonumber(spawn_number), 1, 2)

        spawn_sync_object(machines[machine_number].bhv, machines[machine_number].model, o.oPosX, o.oPosY, o.oPosZ, function(obj)
            obj.oFaceAngleYaw = o.oFaceAngleYaw
            obj.oMoveAngleYaw = o.oFaceAngleYaw
        end)

        obj_mark_for_deletion(o)
    end
end

local function bhv_island_wega_machine_spawner_loop(o)
    if gNetworkPlayers[0].currAreaSyncValid then
        spawn_sync_object(id_bhvWegaMachine3, E_MODEL_WEGA_MACHINE_3, o.oPosX, o.oPosY, o.oPosZ, function(obj)
            obj.oFaceAngleYaw = o.oFaceAngleYaw
            obj.oMoveAngleYaw = o.oFaceAngleYaw
        end)

        obj_mark_for_deletion(o)
    end
end

local function bhv_monitor_init(o)
    o.oFlags = OBJ_FLAG_UPDATE_GFX_POS_AND_ANGLE
	o.header.gfx.skipInViewCheck = true

	cur_obj_scale(1.5)
end

local function bhv_monitor_loop(o)
    o.oFlags = OBJ_FLAG_UPDATE_GFX_POS_AND_ANGLE
	o.header.gfx.skipInViewCheck = true

    if o.oAction == 0 then
        if (o.oTimer % 15) == 0 then
            if o.oAnimState == 2 then
                o.oAnimState = 0
            else
                o.oAnimState = o.oAnimState + 1
            end
        end

        if o.oTimer >= (1.60 * 30) then
            o.oAnimState = 3
            o.oAction = 1
        end
    elseif o.oAction == 1 then
        if (o.oTimer % 10) == 0 then
            if o.oAnimState == 3 then
                o.oAnimState = 4
            else
                o.oAnimState = 3
            end
        end
    end
end

local function bhv_break_door_init(o)
    o.oFlags = OBJ_FLAG_UPDATE_GFX_POS_AND_ANGLE
	o.header.gfx.skipInViewCheck = true

    obj_set_hitbox_radius_and_height(o, 16, 64)

	cur_obj_scale(1)
end

local function bhv_break_door_loop(o)
    o.oFlags = OBJ_FLAG_UPDATE_GFX_POS_AND_ANGLE
	o.header.gfx.skipInViewCheck = true

    if obj_get_first_with_behavior_id(id_bhvWega) then
        if obj_check_hitbox_overlap(o, obj_get_first_with_behavior_id(id_bhvWega)) and o.oAnimState == 0 then
            o.oAnimState = 1

            audio_sample_play(SOUND_EXPLODE_1, {x = o.oPosX, y = o.oPosY, z = o.oPosZ}, 2)

            summon_special_gibs({x = o.oPosX, y = o.oPosY, z = o.oPosZ}, 128)
        end
    end
end

local function bhv_zesty_bar_init(o)
    o.oFlags = OBJ_FLAG_UPDATE_GFX_POS_AND_ANGLE
	o.header.gfx.skipInViewCheck = true
    o.oGravity = 1

    obj_set_hitbox_radius_and_height(o, 64, 64)

	cur_obj_scale(1)
end

local function bhv_zesty_bar_loop(o)
    o.oFlags = OBJ_FLAG_UPDATE_GFX_POS_AND_ANGLE
	o.header.gfx.skipInViewCheck = true

    local m = gMarioStates[0]

    o.oFaceAngleYaw = calculate_yaw({x = o.oPosX, y = o.oPosY, z = o.oPosZ}, gLakituState.pos) - 32768

    if o.oAction == 0 then
        if obj_check_hitbox_overlap(o, m.marioObj) then
            audio_sample_play(SOUND_ZESTY_BAR, {x = o.oPosX, y = o.oPosY, z = o.oPosZ}, 2)
            custom_mario_state_fields.sprintTimer = 256
            obj_mark_for_deletion(o)
        end
    elseif o.oAction == 1 then
        object_step_without_floor_orient()

        if o.oFloorHeight == o.oPosY then
            play_sound(SOUND_GENERAL_BOX_LANDING, o.header.gfx.cameraToObject)
            o.oAction = 0
        end
    end
end

local function bhv_fog_init(o)
    o.oFlags = OBJ_FLAG_UPDATE_GFX_POS_AND_ANGLE
    o.header.gfx.skipInViewCheck = true

	cur_obj_scale(1)
end

local function bhv_fog_loop(o)
    local l = gLakituState

    o.oPosX = l.pos.x
    o.oPosY = l.pos.y + 600
    o.oPosZ = l.pos.z

    o.oFaceAngleYaw = l.yaw
end

local function bhv_retro_pipe_init(o)
    o.oFlags = OBJ_FLAG_UPDATE_GFX_POS_AND_ANGLE
	o.header.gfx.skipInViewCheck = true

    obj_set_hitbox_radius_and_height(o, 128, 128)

	cur_obj_scale(0.5)
end

local function bhv_retro_pipe_loop(o)
    o.oFlags = OBJ_FLAG_UPDATE_GFX_POS_AND_ANGLE
	o.header.gfx.skipInViewCheck = true

    local m = gMarioStates[0]

    o.oFaceAngleYaw = calculate_yaw({x = o.oPosX, y = o.oPosY, z = o.oPosZ}, gLakituState.pos) - 32768

    if obj_check_hitbox_overlap(m.marioObj, o) then
        if not check_if_cutscene_played(1) then
            cutscene_start(1)
        else
            cutscene_start(2)
        end

        obj_mark_for_deletion(o)
    end
end

local function bhv_camera_pos_loop(o)
    local lakitu = gLakituState

    local focus_x = o.oPosX + (20 * sins(o.oMoveAngleYaw) * coss(o.oMoveAnglePitch))
    local focus_y = o.oPosY + (-20 * sins(o.oMoveAnglePitch))
    local focus_z = o.oPosZ + (20 * coss(o.oMoveAngleYaw) * coss(o.oMoveAnglePitch))


    lakitu.pos.x = o.oPosX
    lakitu.pos.y = o.oPosY
    lakitu.pos.z = o.oPosZ

    vec3f_set(lakitu.focus, focus_x, focus_y, focus_z)
    vec3f_set(lakitu.goalFocus, focus_x, focus_y, focus_z)
    vec3f_set(lakitu.curFocus, focus_x, focus_y, focus_z)
end

local function bhv_troll_bomb_init(o)
    o.oFlags = OBJ_FLAG_UPDATE_GFX_POS_AND_ANGLE
	o.header.gfx.skipInViewCheck = true

    o.oMoveAnglePitch = 16384

    start_anim_default(o, 'troll_bomb_spin_anim')

    obj_set_hitbox_radius_and_height(o, 48, 48)

	cur_obj_scale(0.25)
end

local function bhv_troll_bomb_loop(o)
    if o.oAction ~= 8 then
        local m = nearest_mario_state_to_object(o)

        obj_rotate_towards_point(o, m.pos, 5, 5, 3, 6)

        o.oForwardVel = 96 + (o.oTimer / 5)

        obj_compute_vel_from_move_pitch(o.oForwardVel)
        object_step_without_floor_orient()
        cur_obj_update_floor_and_walls()

        if o.oFloorHeight >= o.oPosY - 5 or obj_check_hitbox_overlap(m.marioObj, o) then
            local random_bomb_sound = math.random(1, 3)

            if random_bomb_sound == 1 then
                audio_sample_play(SOUND_TROLL_BOMB_EXPLODE_1, {x = o.oPosX, y = o.oPosY, z = o.oPosZ}, 0.15)
            elseif random_bomb_sound == 2 then
                audio_sample_play(SOUND_TROLL_BOMB_EXPLODE_2, {x = o.oPosX, y = o.oPosY, z = o.oPosZ}, 0.15)
            elseif random_bomb_sound == 3 then
                audio_sample_play(SOUND_TROLL_BOMB_EXPLODE_3, {x = o.oPosX, y = o.oPosY, z = o.oPosZ}, 0.15)
            end

            spawn_non_sync_object(id_bhvExplosion, E_MODEL_EXPLOSION, o.oPosX, o.oPosY, o.oPosZ, nil)

            obj_mark_for_deletion(o)
        end
    end
end

id_bhvBlueShine = hook_behavior(nil, OBJ_LIST_GENACTOR, true, bhv_blue_shine_init, bhv_blue_shine_loop, 'bhvBlueShine')
id_bhvWegaDoor = hook_behavior(nil, OBJ_LIST_SURFACE, true, bhv_wega_door_init, bhv_wega_door_loop, 'bhvWegaDoor')
id_bhvWegaWarp = hook_behavior(nil, OBJ_LIST_SURFACE, true, bhv_wega_warp_init, bhv_wega_warp_loop, 'bhvWegaWarp')
id_bhvWegaMachine1 = hook_behavior(nil, OBJ_LIST_SURFACE, true, bhv_wega_machine_1_init, bhv_wega_machine_1_loop, 'bhvWegaMachine1')
id_bhvWegaMachine2 = hook_behavior(nil, OBJ_LIST_SURFACE, true, bhv_wega_machine_2_init, bhv_wega_machine_2_loop, 'bhvWegaMachine2')
id_bhvWegaMachine3 = hook_behavior(nil, OBJ_LIST_SURFACE, true, bhv_wega_machine_3_init, bhv_wega_machine_3_loop, 'bhvWegaMachine3')
id_bhvTrollBomb = hook_behavior(nil, OBJ_LIST_SURFACE, true, bhv_troll_bomb_init, bhv_troll_bomb_loop, 'bhvTrollBomb')
id_bhvRetroPipe = hook_behavior(nil, OBJ_LIST_SURFACE, true, bhv_retro_pipe_init, bhv_retro_pipe_loop, 'bhvRetroPipe')
id_bhvFog = hook_behavior(nil, OBJ_LIST_SURFACE, true, bhv_fog_init, bhv_fog_loop, 'bhvFog')
id_bhvMonitor = hook_behavior(nil, OBJ_LIST_SURFACE, true, bhv_monitor_init, bhv_monitor_loop, 'bhvMonitor')
id_bhvBreakDoor = hook_behavior(nil, OBJ_LIST_SURFACE, true, bhv_break_door_init, bhv_break_door_loop, 'bhvBreakDoor')
id_bhvMachineGib = hook_behavior(nil, OBJ_LIST_SURFACE, true, machine_gib_init, machine_gib_loop, 'bhvMachineGib')
id_bhvZestyBar = hook_behavior(nil, OBJ_LIST_SURFACE, true, bhv_zesty_bar_init, bhv_zesty_bar_loop, 'bhvZestyBar')
id_bhvWegaMachineSpawner = hook_behavior(nil, OBJ_LIST_SURFACE, true, nil, bhv_wega_machine_spawner_loop, 'bhvWegaMachineSpawner')
id_bhvIslandWegaMachineSpawner = hook_behavior(nil, OBJ_LIST_SURFACE, true, nil, bhv_island_wega_machine_spawner_loop, 'bhvIslandWegaMachineSpawner')
id_bhvCameraPosition = hook_behavior(nil, OBJ_LIST_SURFACE, true, nil, bhv_camera_pos_loop, 'bhvCameraPosition')

-- Non-obj functions, for example spawning stuff

local function spawn_troll_bomb_clusters()
    local cluster_amount = 0
    local m = gMarioStates[0]

    repeat
        local cluster_position_x = math.random(-4000, 4000)
        local cluster_position_y = math.random(1000, 4000)
        local cluster_position_z = math.random(-4000, 4000)

        local bomb_amount = 0

        repeat
            spawn_sync_object(id_bhvTrollBomb, E_MODEL_TROLL_BOMB, cluster_position_x + math.random(-200, 200), cluster_position_y + math.random(-200, 600), cluster_position_z + math.random(-200, 80), nil)

            bomb_amount = bomb_amount + 1
        until bomb_amount == 2

        cluster_amount = cluster_amount + 1
    until cluster_amount == 3

    audio_sample_play(SOUND_TROLL_BOMB_SPAWN, m.pos, 0.15)
end

local function spawn_zesty_bars()
    local random_pos_x = math.random(-2200, 2200)
    local random_pos_z = math.random(-2200, 2200)

    spawn_sync_object(id_bhvZestyBar, E_MODEL_ZESTY_BAR, random_pos_x, 250, random_pos_z, function(o)
        o.oAction = 1
    end)
end

local function level_obj_update()
    local n = gNetworkPlayers[0]

    if n.currLevelNum == LEVEL_WF and (level_timer % (15 * 30)) == 0 and obj_get_first_with_behavior_id(id_bhvMaltigi) then
        spawn_troll_bomb_clusters()
        spawn_zesty_bars()
    end
end

hook_event(HOOK_UPDATE, level_obj_update)