if not is_game_single_player then return end

in_menu = true
local menu_state = 1
local button_func
local function_timer = -1
local current_song
local music_data

local TEX_EXCLAMATION_BLOCK = get_texture_info("box")
local TEX_YELLOW_EXCLAMATION_BLOCK = get_texture_info("yellow_box")
local TEX_GRAY_EXCLAMATION_BLOCK = get_texture_info("gray_box")
local TEX_TITLE = get_texture_info("wegaswansion")
local TEX_FADE = get_texture_info("fade")
local TEX_CURSOR = get_texture_info("cursor")
local prev_cursor = {x = 0, y = 0}

local GOLDEN_SIGMA_1 = get_texture_info("golden_sigma_1")
local GOLDEN_SIGMA_2 = get_texture_info("golden_sigma_2")
local GOLDEN_SIGMA_3 = get_texture_info("golden_sigma_3")
local GOLDEN_SIGMA_4 = get_texture_info("golden_sigma_4")
local GOLDEN_SIGMA_5 = get_texture_info("golden_sigma_5")

local width = djui_hud_get_screen_width()
local height = djui_hud_get_screen_height()

local character = math.random(1, 2)

local char_tex
local char_timer = 0

local current_display_character = 1

-- Custom functions for Menu States

local function mouse_over_button(x, y, hitbox_x, hitbox_y)
    local mouse_x = djui_hud_get_mouse_x()
    local mouse_y = djui_hud_get_mouse_y()

    local x_limit = (x + hitbox_x) + 32
    local y_limit = (y + hitbox_y)

    if mouse_x < x_limit and mouse_x > x + 16 and mouse_y < y_limit and mouse_y > y - 16 then
        return true
    end

    return false
end

local function render_button(string, x, y, transition, locked, func) -- y actually acts as the centered y, not the top y
    if transition then
        if not locked then
            if mouse_over_button(x, y, djui_hud_measure_text(string) * 2, 48) then
                djui_hud_set_color(255, 255, 255, 255)
                djui_hud_print_text(string, x + 32, y - 24, 2)
                button_func = func
            else
                djui_hud_set_color(165, 165, 165, 255)
                djui_hud_print_text(string, x + 32, y - 24, 2)

                if button_func == func and function_timer == -1 then
                    button_func = nil
                end
            end
        else
            djui_hud_set_color(165, 165, 165, 255)
            djui_hud_print_text(string .. ' [Locked]', x + 32, y - 24, 2)
        end

        djui_hud_print_text(string, x + 32, y - 24, 2)
    else
        if mouse_over_button(x, y, djui_hud_measure_text(string) * 2, 48) then
            djui_hud_set_color(255, 255, 255, 255)
            djui_hud_print_text(string, x + 32, y - 24, 2)

            if (gMarioStates[0].controller.buttonPressed & A_BUTTON) ~= 0 then
                func()
            end
        else
            djui_hud_set_color(165, 165, 165, 255)
            djui_hud_print_text(string, x + 32, y - 24, 2)
        end

        djui_hud_print_text(string, x + 32, y - 24, 2)
    end
end

local function menu_state_1()
    if character == 1 then
        local random = math.random(1, 32)
        local tex = GOLDEN_SIGMA_1

        if char_tex == nil then
            if random <= 31 then
                char_tex = TEX_WEGA_JUMPSCARE
                tex = TEX_WEGA_JUMPSCARE
                char_timer = 10
            elseif random == 32 then
                char_tex = TEX_MALTIGA_JUMPSCARE
                tex = TEX_MALTIGA_JUMPSCARE
                char_timer = 2
            end
        else
            tex = char_tex

            if char_timer == 0 then
                char_tex = nil
            end
        end

        if char_timer ~= 0 then
            char_timer = char_timer - 1
        end

        local sigma_scale = (height / tex.height) * 1
        local random_2 = math.random(190, 200)

        djui_hud_set_color(random_2, random_2, random_2, 255)

        djui_hud_render_texture(tex, width - (tex.width * sigma_scale), 0, sigma_scale, sigma_scale)
    elseif character == 2 then
        local random = math.random(1, 128)
        local tex = GOLDEN_SIGMA_1

        if char_tex == nil then
            if random <= 124 then
                char_tex = GOLDEN_SIGMA_1
                tex = GOLDEN_SIGMA_1
                char_timer = 10
            elseif random == 125 then
                char_tex = GOLDEN_SIGMA_2
                tex = GOLDEN_SIGMA_2
                char_timer = 5
            elseif random == 126 then
                char_tex = GOLDEN_SIGMA_3
                tex = GOLDEN_SIGMA_3
                char_timer = 5
            elseif random == 127 then
                char_tex = GOLDEN_SIGMA_4
                tex = GOLDEN_SIGMA_4
                char_timer = 5
            elseif random == 128 then
                char_tex = GOLDEN_SIGMA_5
                tex = GOLDEN_SIGMA_5
                char_timer = 5
            end
        else
            tex = char_tex

            if char_timer == 0 then
                char_tex = nil
            end
        end

        if char_timer ~= 0 then
            char_timer = char_timer - 1
        end

        local sigma_scale = (height / tex.height) * 1
        local random_2 = math.random(190, 200)

        djui_hud_set_color(random_2, random_2, random_2, 255)

        djui_hud_render_texture(tex, width - (tex.width * sigma_scale), 0, sigma_scale, sigma_scale)
    end

    djui_hud_set_color(255, 255, 255, 255)

    djui_hud_print_text('WEGA\'S WANSION', 16, 16, 4)

    current_song = MAIN_MENU
end

local function exit_menu()
    local m = gMarioStates[0]

    custom_mario_state_fields.inactive = false
    in_menu = false
    m.freeze = false
end

local function start_game()
    exit_menu()
    warp_to_level(LEVEL_DDD, 1, 1)
end

local function init_menu()
    play_transition(WARP_TRANSITION_FADE_FROM_COLOR, 10, 0, 0, 0)
    button_func = nil
end

local function open_extras_menu()
    init_menu()
    menu_state = 2

    stop_all_music()
    audio_stream_play(EXTRAS_MENU, true, 1)
    current_song = EXTRAS_MENU
end

local function open_normal_menu()
    init_menu()
    menu_state = 1

    character = math.random(1, 2)

    char_tex = nil
    char_timer = 0

    stop_all_music()
    audio_stream_play(MAIN_MENU, false, 1)
    current_song = MAIN_MENU
end

local function open_extras_music_menu()
    init_menu()
    menu_state = 3

    stop_all_music()
end

local function menu_state_3()
    local music_track = 1
    local m = gMarioStates[0]

    for i = 1, #music_table do
        local music = music_table[i]

        if music.display_info ~= nil then
            local mouse_hovering = mouse_over_button(32, (64 * music_track), djui_hud_measure_text(music.display_info.name) * 2, 48)

            if mouse_hovering then
                djui_hud_set_color(255, 255, 255, 255)
            else
                djui_hud_set_color(180, 180, 180, 255)
            end

            djui_hud_print_text(music.display_info.name, 32, (64 * music_track) - 24, 2)

            if mouse_hovering and (m.controller.buttonPressed & A_BUTTON) ~= 0 then
                stop_all_music()
                audio_stream_play(music.song, true, 1)
                music_data = music_table[i].display_info
                current_song = music.song
            end

            music_track = music_track + 1
        end
    end

    djui_hud_set_color(255, 255, 255, 255)

    if music_data ~= nil then
        djui_hud_print_text(music_data.name, width / 2 - (djui_hud_measure_text(music_data.name) * 2), height - 256, 4)
        djui_hud_print_text(music_data.desc, width / 2 - djui_hud_measure_text(music_data.desc), height - 128, 2)
    end
end

local function open_extras_cast_menu()
    init_menu()
    menu_state = 4

    stop_all_music()
    audio_stream_play(EXTRAS_MENU, false, 1)
    current_song = EXTRAS_MENU
end

local function menu_state_4()
    local current_char = 1
    local m = gMarioStates[0]

    local character_table = {
        [1] = {
            name = 'Wega', desc = 'The man, the myth, the legend.', id = id_bhvWega, model = E_MODEL_WEGA, y_offset = 0
        },
        [2] = {
            name = 'Metal Wega', desc = 'He\'s not so common, but he\'ll make his presence known!', id = id_bhvRunningWega, model = E_MODEL_RUNNING_WEGA, y_offset = 0
        },
        [3] = {
            name = 'Maltiga', desc = 'Don\'t anger him more!', id = id_bhvMaltigi, model = E_MODEL_MALTIGI, y_offset = 0
        },
        [4] = {
            name = 'Golden Sigma', desc = 'Quite the sigma!', id = id_bhvGoldenSigma, model = E_MODEL_GOLDEN_SIGMA, y_offset = 256
        },
        [5] = {
            name = 'Trollface', desc = 'Trollface has had a long journey, from Flood Expanded, to here!', id = id_bhvTrollface, model = E_MODEL_TROLLFACE, y_offset = 256
        },
        [6] = {
            name = 'Trollbomb', desc = 'Trollface\'s distant relative, They\'re quite the nuisance!', id = id_bhvTrollBomb, model = E_MODEL_TROLL_BOMB, y_offset = 256
        },
    }

    for i = 1, #character_table do
        local char = character_table[i]

        local mouse_hovering = mouse_over_button(32, (64 * current_char), djui_hud_measure_text(char.name) * 2, 48)

        if mouse_hovering then
            djui_hud_set_color(255, 255, 255, 255)
        else
            djui_hud_set_color(180, 180, 180, 255)
        end

        djui_hud_print_text(char.name, 32, (64 * current_char) - 24, 2)

        if mouse_hovering and (m.controller.buttonPressed & A_BUTTON) ~= 0 then
            current_display_character = i
        end

        current_char = current_char + 1
    end

    djui_hud_set_color(255, 255, 255, 255)

    if current_display_character ~= nil then
        djui_hud_print_text(character_table[current_display_character].name, width / 2 - (djui_hud_measure_text(character_table[current_display_character].name) * 2), height - 256, 4)
        djui_hud_print_text(character_table[current_display_character].desc, width / 2 - djui_hud_measure_text(character_table[current_display_character].desc), height - 128, 2)
    end

    if display_character == nil then
        display_character = spawn_non_sync_object(character_table[current_display_character].id, character_table[current_display_character].model, 0, 64 + character_table[current_display_character].y_offset, 0, function(obj)
            obj.oAction = 8
        end)
    else
        if get_id_from_behavior(display_character.behavior) ~= character_table[current_display_character].id then
            obj_mark_for_deletion(display_character)

            display_character = spawn_non_sync_object(character_table[current_display_character].id, character_table[current_display_character].model, 0, 64 + character_table[current_display_character].y_offset, 0, function(obj)
                obj.oAction = 8
            end)
        end
    end

    if display_character then
        display_character.oFaceAngleYaw = display_character.oFaceAngleYaw + 256
    end

    gMarioStates[0].marioObj.header.gfx.node.flags = gMarioStates[0].marioObj.header.gfx.node.flags & ~GRAPH_RENDER_ACTIVE

    set_override_fov(75)
end

local menu_states = {
    [1] = {
        function() menu_state_1() end,
        function() render_button('Start Game', 5, (64 * 4) - 24, true, false, start_game) end,
        function() render_button('Reset Save', 5, (64 * 6) - 24, false, false, clear_mod_storage) end,
        function() render_button('Extras', 5, (64 * 8) - 24, true, not big_events_table.maltiga_defeated, open_extras_menu) end
    },
    [2] = {
        function() render_button('Music', 5, (64 * 1) - 24, true, false, open_extras_music_menu) end,
        function() render_button('Cast', 5, (64 * 2) - 24, true, false, open_extras_cast_menu) end,
        function() render_button('Back', 5, (64 * 3) - 24, true, false, open_normal_menu) end
    },
    [3] = {
        function() menu_state_3() end,
        function() render_button('Back', 5, height - 64, true, false, open_extras_menu) end
    },
    [4] = {
        function() menu_state_4() end,
        function() render_button('Back', 5, height - 64, true, false, open_extras_menu) end
    },
}

local function mario_update(m)
    if m ~= gMarioStates[0] then return end

    if in_menu then
        custom_mario_state_fields.inactive = true

        m.freeze = true

        m.pos.x = 0
        m.pos.y = 500
        m.pos.z = 0

        set_mario_action(m, ACT_FREEFALL, 1)

        if gNetworkPlayers[0].currLevelNum ~= LEVEL_JRB then
            warp_to_level(LEVEL_JRB, 1, 1)
        end

        if (m.controller.buttonPressed & A_BUTTON) ~= 0 and button_func then
            function_timer = 10
            play_transition(WARP_TRANSITION_FADE_INTO_COLOR, 10, 0, 0, 0)
            fade_out_stream(current_song, 10)
        end

        if function_timer == 0 and button_func then
            button_func()
        end
    end
end

local function render_menu()
    if not in_menu then return end

    if menu_state == 1 then
        if character == 1 then
            djui_hud_set_color((135 / 4), (35 / 4), (255 / 4), 255)

            djui_hud_render_rect(0, 0, width + 5, height + 5)

            djui_hud_set_color(255, 255, 255, 15)

            for i = 0, math.ceil(width / 30) do
                for j = 0, 10 do
                    djui_hud_render_texture(TEX_EXCLAMATION_BLOCK, i * 128 + ((get_global_timer() % 64) * 8) - 512, j * 128, 4, 4)
                end
            end
        else
            djui_hud_set_color((255 / 4), (235 / 4), (35 / 4), 255)

            djui_hud_render_rect(0, 0, width + 5, height + 5)

            djui_hud_set_color(255, 255, 255, 15)

            for i = 0, math.ceil(width / 30) do
                for j = 0, 10 do
                    djui_hud_render_texture(TEX_YELLOW_EXCLAMATION_BLOCK, i * 128 + ((get_global_timer() % 64) * 8) - 512, j * 128, 4, 4)
                end
            end
        end
    elseif menu_state ~= 4 then
        djui_hud_set_color((128 / 4), (128 / 4), (128 / 4), 255)

        djui_hud_render_rect(0, 0, width + 5, height + 5)

        djui_hud_set_color(255, 255, 255, 15)

        for i = 0, math.ceil(width / 30) do
            for j = 0, 10 do
                djui_hud_render_texture(TEX_GRAY_EXCLAMATION_BLOCK, i * 128 + ((get_global_timer() % 64) * 8) - 512, j * 128, 4, 4)
            end
        end
    end

    djui_hud_set_color(255, 255, 255, 255)

    if menu_state ~= 4 then
        djui_hud_render_texture(TEX_FADE, 0, 0, width / 32, height / 32)
    end

    for i = 1, #menu_states[menu_state] do
        menu_states[menu_state][i]()
    end

    if function_timer > -1 then
        function_timer = function_timer - 1
    end

    djui_hud_set_color(255, 255, 255, 255)

    djui_hud_render_texture_interpolated(TEX_CURSOR, prev_cursor.x, prev_cursor.y, 2, 2, djui_hud_get_mouse_x(), djui_hud_get_mouse_y(), 2, 2)
    prev_cursor.x = djui_hud_get_mouse_x()
    prev_cursor.y = djui_hud_get_mouse_y()
end

hook_event(HOOK_MARIO_UPDATE, mario_update)
hook_event(HOOK_ON_HUD_RENDER_BEHIND, render_menu)